@@ -12,6 +12,8 @@ from cloudvolume.lib import Bbox, yellow
 # Offset = Tuple[int, int, int]
 from .validate import validate_by_template_matching
 
+# from memory_profiler import profile
+
 
 class Chunk(NDArrayOperatorsMixin):
     r"""
@@ -87,9 +89,10 @@ class Chunk(NDArrayOperatorsMixin):
         return cls(chunk, global_offset=voxel_offset)
 
     @classmethod
-    def from_tif(cls, file_name: str, global_offset: tuple=None):
+    def from_tif(cls, file_name: str, global_offset: tuple=None, dtype: str = None):
         arr = tifffile.imread(file_name)
-
+        if dtype:
+            arr = arr.astype(dtype)
         return cls(arr, global_offset=global_offset)
     
     def to_tif(self, file_name: str=None, global_offset: tuple=None):
@@ -109,28 +112,71 @@ class Chunk(NDArrayOperatorsMixin):
 
     @classmethod
     def from_h5(cls, file_name: str,
-                dataset_path: str = '/main',
-                global_offset: tuple = None):
+                voxel_offset: tuple=None,
+                dataset_path: str = None,
+                cutout_start: tuple=None,
+                cutout_stop: tuple=None,
+                cutout_size: tuple=None):
 
         assert os.path.exists(file_name)
         assert h5py.is_hdf5(file_name)
 
-        print('read from HDF5 file: {}'.format(file_name))
-
-        global_offset_path = os.path.join(os.path.dirname(file_name),
+        voxel_offset_path = os.path.join(os.path.dirname(file_name),
                                        'global_offset')
         with h5py.File(file_name, 'r') as f:
-            arr = np.asarray(f[dataset_path])
+            if dataset_path is None:
+                for key in f.keys():
+                    if 'offset' not in key:
+                        # the first name without offset inside
+                        dataset_path = key
+            dset = f[dataset_path]
+            if voxel_offset is None: 
+                if voxel_offset_path in f:
+                    voxel_offset = tuple(f[voxel_offset_path])
+                else:
+                    voxel_offset = tuple(0 for _ in range(dset.ndim))
 
-            if global_offset is None:
-                if global_offset_path in f:
-                    global_offset = tuple(f[global_offset_path])
+            if cutout_start is None:
+                cutout_start = voxel_offset
 
-        print('global offset: {}'.format(global_offset))
+            for c, v in zip(cutout_start, voxel_offset):
+                assert c >= v, "can only cutout after the global voxel offset."
 
-        return cls(arr, global_offset=global_offset)
+            if cutout_start is not None:
+                if cutout_stop is None:
+                    if cutout_size is None:
+                        cutout_size = dset.shape
+                    cutout_stop = tuple(t+s for t, s in zip(cutout_start, cutout_size))
 
-    def to_h5(self, file_name: str):
+                if dset.ndim == 3:
+                    dset = dset[
+                        cutout_start[0]-voxel_offset[0]:cutout_stop[0]-voxel_offset[0],
+                        cutout_start[1]-voxel_offset[1]:cutout_stop[1]-voxel_offset[1],
+                        cutout_start[2]-voxel_offset[2]:cutout_stop[2]-voxel_offset[2],
+                    ]
+                elif dset.ndim == 4:
+                    assert len(cutout_start) == 4
+                    assert len(cutout_stop) == 4
+                    dset = dset[
+                        cutout_start[0]-voxel_offset[0]:cutout_stop[0]-voxel_offset[0],
+                        cutout_start[1]-voxel_offset[1]:cutout_stop[1]-voxel_offset[1],
+                        cutout_start[2]-voxel_offset[2]:cutout_stop[2]-voxel_offset[2],
+                        cutout_start[3]-voxel_offset[3]:cutout_stop[3]-voxel_offset[3],
+                    ]
+        
+        print('read from HDF5 file: {} and start with {}, ends with {}, size is {}'.format(
+            file_name, cutout_start, cutout_stop, cutout_size))
+        arr = np.asarray(dset)
+        if arr.dtype == np.dtype('<f4'):
+            arr = arr.astype('float32')
+        elif arr.dtype == np.dtype('<f8'):
+            arr = arr.astype('float64') 
+
+        print('new chunk voxel offset: {}'.format(cutout_start))
+
+        return cls(arr, global_offset=cutout_start)
+
+    def to_h5(self, file_name: str, with_offset: bool=True, compression="gzip"):
         assert '.h5' in file_name
 
         print('write chunk to file: ', file_name)
@@ -138,8 +184,9 @@ class Chunk(NDArrayOperatorsMixin):
             os.remove(file_name)
 
         with h5py.File(file_name, 'w') as f:
-            f.create_dataset('/main', data=self.array)
-            f.create_dataset('/global_offset', data=self.global_offset)
+            f.create_dataset('/main', data=self.array, compression=compression)
+            if with_offset:
+                f.create_dataset('/global_offset', data=self.global_offset)
     
     def __array__(self):
         return self.array
@@ -249,10 +296,10 @@ class Chunk(NDArrayOperatorsMixin):
             return self
     
     def max(self, *args, **kwargs):
-        return self.array.max(args, kwargs)
+        return self.array.max(*args, **kwargs)
 
     def min(self, *args, **kwargs):
-        return self.array.min(args, kwargs)
+        return self.array.min(*args, **kwargs)
 
     def transpose(self):
         """To-Do: support arbitrary axis transpose"""
@@ -270,7 +317,8 @@ class Chunk(NDArrayOperatorsMixin):
         arr = np.squeeze(self, axis=0)
         global_offset = self.global_offset[:axis] + self.global_offset[axis+1:]
         return Chunk(arr, global_offset=global_offset)
-
+    
+    # @profile(precision=0)
     def channel_voting(self):
         assert self.ndim == 4
         assert self.shape[0] <= 256
@@ -281,8 +329,6 @@ class Chunk(NDArrayOperatorsMixin):
         return Chunk(out, global_offset=self.global_offset[1:])
 
     def mask_using_last_channel(self, threshold: float = 0.3) -> np.ndarray:
-        assert self.ndim==4
-
         mask = (self.array[-1, :, :, :] < threshold)
         ret = self.array[:-1, ...]
         ret *= mask
@@ -313,13 +359,25 @@ class Chunk(NDArrayOperatorsMixin):
             assert output_bbox is not None
             return self.cutout(output_bbox.to_slices())
     
+    def threshold(self, threshold: float):
+        seg = self > threshold
+        if seg.ndim == 4:
+            global_offset = self.global_offset
+            assert seg.shape[0] == 1
+            seg = seg.array[0, ...]
+            global_offset = global_offset[1:]
+            seg = Chunk(seg, global_offset = global_offset)
+        # neuroglancer do not support bool datatype
+        # numpy store bool as uint8 datatype, so this will not increase size.
+        seg = seg.astype(np.uint8)
+        return seg
+    
     def connected_component(self, threshold: float = 0.5, 
                             connectivity: int = 26):
         """threshold the map chunk and get connected components."""
-        global_offset = self.global_offset
-        seg = self > threshold
+        seg = self.threshold(threshold)
         seg = cc3d.connected_components(seg.array, connectivity=connectivity)
-        return Chunk(seg, global_offset=global_offset)
+        return Chunk(seg, global_offset=self.global_offset)
 
     def where(self, mask: np.ndarray) -> tuple:
         """
