@@ -47,7 +47,6 @@ class Inferencer(object):
                  mask_myelin_threshold = None,
                  dry_run: bool = False,
                  verbose: int = 1):
-        
         assert input_size is None or patch_num is None 
         
         if output_patch_size is None:
@@ -183,8 +182,8 @@ class Inferencer(object):
             from .patch.pytorch_multitask import PyTorchMultitask as PatchInferencer
         elif framework == 'identity':
             from .patch.identity import Identity as PatchInferencer
-        elif framework == 'general':
-            from .patch.general import General as PatchInferencer
+        elif framework == 'universal':
+            from .patch.universal import Universal as PatchInferencer
         else:
             raise Exception(f'invalid inference backend: {self.framework}')
         
@@ -284,7 +283,7 @@ class Inferencer(object):
             patch_mask = Chunk(self.patch_inferencer.output_patch_mask_numpy,
                                global_offset=patch_global_offset)
             self.output_chunk_mask.blend(patch_mask)
-        
+
         # normalize weight, so accumulated inference result multiplies
         # this mask will result in 1
         self.output_chunk_mask.array = 1.0 / self.output_chunk_mask.array
@@ -340,7 +339,11 @@ class Inferencer(object):
        
         if input_chunk == 0:
             print('input is all zero, return zero buffer directly')
-            return output_buffer
+            if self.mask_myelin_threshold:
+                assert output_buffer.shape[0] == 4
+                return output_buffer[:-1, ...]
+            else:
+                return output_buffer
         
         if np.issubdtype(input_chunk.dtype, np.integer):
             # normalize to 0-1 value range
@@ -352,7 +355,7 @@ class Inferencer(object):
 
         # iterate the offset list
         for i in tqdm(range(0, len(self.patch_slices_list), self.batch_size),
-                      disable=not self.verbose,
+                      disable=(self.verbose <= 0),
                       desc='ConvNet inference for patches: '):
             if self.verbose:
                 start = time.time()
@@ -395,7 +398,6 @@ class Inferencer(object):
                 #        bbox.minpt[-2]<81545 and bbox.maxpt[-2]>81545 and \
                 #        bbox.minpt[-3]<17298 and bbox.maxpt[-3]>17298:
                 #    print('save patch: ', output_chunk.bbox)
-                #    breakpoint()
                 #    output_chunk.to_tif()
                 #    #input_chunk.cutout(slices[0]).to_tif()
 
@@ -423,8 +425,7 @@ class Inferencer(object):
             output_chunk = output_buffer.mask_using_last_channel(
                 threshold = self.mask_myelin_threshold)
 
-            # currently neuroglancer only support float32, not float16
-            if output_chunk.dtype == np.dtype('float16'):
+            if output_chunk.dtype == np.dtype('<f4'):
                 output_chunk = output_chunk.astype('float32')
 
             return output_chunk
